#include <math.h>
#define PI 3.141592653589793238462643
/* TEST STATISTICS */
void mdh0w1(double *y, int *n, int *m, double *a, double *t) 
{
  int it, is, l;
  double q, iwx, d;
  /* scaling by a */
  for(l = 0; l < *n; l++) 
    {
      y[l] = y[l] / *a;
    }
  /* starting value */
  q = 0.0 ; 
  /* calculate the sum */
  for(it = *m; it < (*n - 1) ; it++) 
    { 
      q += y[it] * y[it];
      for(is = (it+1); is < *n ; is++)
	{
	  iwx = 1.0;
	  for(l = 1; l < (*m + 1) ; l++)
	    {
	      d = y[it - l] - y[is - l];
	      iwx = iwx / (1.0 + (d * d));
	    }
	  q += 2.0 * y[it] * y[is] * iwx;
	}
    }
  q += y[*n - 1] * y[*n - 1];
  *t = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * q / *n; 
}
/* add WILD BOOTSTRAP and P-VALUE - puvodni verze */
void mdh0w1wb(double *y, int *n, int *m, double *a, int *b, double *eta, double *t) 
{
  int it, is, l, ib, count;
  double q, iwx, d, tb;
  /* scaling by a */
  for(l = 0; l < *n; l++) 
    {
      y[l] = y[l] / *a;
    }
  /* starting value */
  q = 0.0 ; 
  /* calculate the sum */
  for(it = *m; it < (*n - 1) ; it++) 
    { 
      q += y[it] * y[it];
      for(is = (it+1); is < *n ; is++)
	{
	  iwx = 1.0;
	  for(l = 1; l < (*m + 1) ; l++)
	    {
	      d = y[it - l] - y[is - l];
	      iwx = iwx / (1.0 + (d * d));
	    }
	  q += 2.0 * y[it] * y[is] * iwx;
	}
    }
  q += y[*n - 1] * y[*n - 1];
  t[0] = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * q / *n; 
  /* WILD BOOTSTRAP */
  count=0;
  for (ib = 0; ib < *b; ib++) {
    /* calculate the test stat in wild bootstrap */
    q = 0.0 ; 
    for(it = *m; it < (*n - 1) ; it++) 
      { 
	q += y[it] * y[it] * eta[ib * *n + it] * eta[ib * *n + it];
	for(is = (it+1); is < *n ; is++)
	  {
	    iwx = 1.0;
	    for(l = 1; l < (*m + 1) ; l++)
	      {
		d = y[it - l] - y[is - l];
		iwx = iwx / (1.0 + (d * d));
	      }
	    q += 2.0 * y[it] * y[is] * eta[ib * *n + it] * eta[ib * *n + is] * iwx;
	  }
      }
    q += y[*n - 1] * y[*n - 1] * eta[ib * *n + *n -1] * eta[ib * *n + *n -1];
    tb = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * q / *n; /* bootstrap test stat */
    if (t[0] < tb) {
      count += 1;
    }
  }
  t[1]= 1.0 * count / *b;
}
/* add WILD BOOTSTRAP and P-VALUE - centrovani y v bootstrapu */
void mdh0w1cwb(double *y, double *yc, int *n, int *m, double *a, int *b, double *eta, double *t) 
{
  int it, is, l, ib, count;
  double q, iwx, d, tb;
  /* scaling by a */
  for(l = 0; l < *n; l++) 
    {
      y[l] = y[l] / *a;
    }
  /* starting value */
  q = 0.0 ; 
  /* calculate the sum */
  for(it = *m; it < (*n - 1) ; it++) 
    { 
      q += y[it] * y[it];
      for(is = (it+1); is < *n ; is++)
	{
	  iwx = 1.0;
	  for(l = 1; l < (*m + 1) ; l++)
	    {
	      d = y[it - l] - y[is - l];
	      iwx = iwx / (1.0 + (d * d));
	    }
	  q += 2.0 * y[it] * y[is] * iwx;
	}
    }
  q += y[*n - 1] * y[*n - 1];
  t[0] = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * q / *n; 
  /* WILD BOOTSTRAP */
  count=0;
  for (ib = 0; ib < *b; ib++) {
    /* calculate the test stat in wild bootstrap */
    q = 0.0 ; 
    for(it = *m; it < (*n - 1) ; it++) 
      { 
	q += yc[it] * yc[it] * eta[ib * *n + it] * eta[ib * *n + it];
	for(is = (it+1); is < *n ; is++)
	  {
	    iwx = 1.0;
	    for(l = 1; l < (*m + 1) ; l++)
	      {
		d = y[it - l] - y[is - l] ; /* ma tu byt eta? */
		iwx = iwx / (1.0 + (d * d));
	      }
	    q += 2.0 * yc[it] * yc[is] * eta[ib * *n + it] * eta[ib * *n + is] * iwx;
	  }
      }
    q += yc[*n - 1] * yc[*n - 1] * eta[ib * *n + *n -1] * eta[ib * *n + *n -1];
    tb = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * q / *n; /* bootstrap test stat */
    if (t[0] < tb) {
      count += 1;
    }
  }
  t[1]= 1.0 * count / *b;
}
