#include <math.h>
#define PI 3.141592653589793238462643
/* TEST STATISTICS */
void mdhs0w1(double *y, int *n, int *m, double *a, double *g, double *t) 
{
  int it, is, l, k;
  double q, iwx, d, qmax;
  double qs[*n];
  /* scaling by a */
  for(l = 0; l < *n; l++) 
    {
      y[l] = y[l] / *a;
    }
  /* sequential weight */
  for(l = 0; l < (*n - *m - 1); l++) 
    {
      qs[l] = 1.0 / exp ( *g * log( ( 1.0 - ( l + *m + 1.0 ) / *n ) ) ) ;
    }
  /* starting value for the sum */
  q = y[*n - 1] * y[*n - 1];
  qmax = q * qs[*n - *m - 2];
  /* calculate the maximum via recursion */
  for (k = *n -3; k > *m -1 ; k -= 1) {
    it = k + 1;
    q += y[it] * y[it];
    for(is = (it+1); is < *n ; is++)
	{
	  iwx = 1.0;
	  for(l = 1; l < (*m + 1) ; l++)
	    {
	      d = y[it - l] - y[is - l];
	      iwx = iwx / (1.0 + (d * d));
	    }
	  q += 2.0 * y[it] * y[is] * iwx;
	}
    if (q * qs[k - *m] > qmax) {
      qmax = q * qs[k - *m];
    }
  }
  *t = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * qmax / *n; 
}
/* add WILD BOOTSTRAP and P-VALUE*/
void mdhs0w1wb(double *y, int *n, int *m, double *a, double *g, int *b, double *eta, double *t) 
{
  int it, is, l, k, ib, count;
  double q, iwx, d, qmax, tb;
  double qs[*n];
  /* scaling by a */
  for(l = 0; l < *n; l++) 
    {
      y[l] = y[l] / *a;
    }
 /* sequential weight */
  for(l = 0; l < (*n - *m - 2); l++) 
    {
      qs[l] = 1.0 / exp ( *g * log( ( 1.0 - ( l + *m + 1.0 ) / *n ) ) ) ;
    }
  /* starting value */
  q = y[*n - 1] * y[*n - 1];
  qmax = q * qs[*n - *m - 2];
  /* calculate the maximum via recursion */
  for (k = *n -3; k > *m -1 ; k -= 1) {
    it = k + 1;
    q += y[it] * y[it];
    for(is = (it+1); is < *n ; is++)
	{
	  iwx = 1.0;
	  for(l = 1; l < (*m + 1) ; l++)
	    {
	      d = y[it - l] - y[is - l];
	      iwx = iwx / (1.0 + (d * d));
	    }
	  q += 2.0 * y[it] * y[is] * iwx;
	}
    if (q * qs[k - *m] > qmax) {
      qmax = q * qs[k - *m];
    }
  }
  t[0] = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * qmax / *n; 
  /* WILD BOOTSTRAP */
  count=0;
  for (ib = 0; ib < *b; ib++) {
    /* calculate the test stat in wild bootstrap */

    q = y[*n - 1] * y[*n - 1] * eta[ib * *n + *n -1] * eta[ib * *n + *n -1];
    qmax = q * qs[*n - *m - 2];
    /* calculate the maximum via recursion */
    for (k = *n -3; k > *m -1 ; k -= 1) {
      it = k + 1;
      q += y[it] * y[it] * eta[ib * *n + it] * eta[ib * *n + it];
      for(is = (it+1); is < *n ; is++)
	{
	  iwx = 1.0;
	  for(l = 1; l < (*m + 1) ; l++)
	    {
	      d = y[it - l] - y[is - l];
	      iwx = iwx / (1.0 + (d * d));
	    }
	  q += 2.0 * y[it] * y[is]  * eta[ib * *n + it] * eta[ib * *n + is] * iwx;
	}
      if (q * qs[k - *m] > qmax) {
	qmax = q * qs[k - *m];
      }
    }
    /* bootstrap test stat */
    tb = (*a * *a) * (exp( 1.0 * *m * log(2.0 / *a))) * qmax / *n; 
    if (t[0] < tb) {
      count += 1;
    }
  }
  t[1]= 1.0 * count / *b;
}
